match x:
    case var: ...  # capture pattern
    case 2: ...
match x:
    case _: ...
    case 2: ...    # wildcard pattern
match x:
    case var1 as var2: ...  # as pattern with irrefutable left-hand side
    case 2: ...
match x:
    case enum.variant | var: ...  # or pattern with irrefutable part
    case 2: ...



# No indentation before the function definition
if call(foo
def bar():
    pass


# Indented function definition
if call(foo
    def bar():
        pass


# There are multiple non-logical newlines (blank lines) in the `if` body
if call(foo


    def bar():
        pass


# There are trailing whitespaces in the blank line inside the `if` body
if call(foo
        
    def bar():
        pass


# The lexer is nested with multiple levels of parentheses
if call(foo, [a, b
    def bar():
        pass


# The outer parenthesis is closed but the inner bracket isn't
if call(foo, [a, b)
    def bar():
        pass


# The parser tries to recover from an unclosed `]` when the current token is `)`. This
# test is to make sure it emits a `NonLogicalNewline` token after `b`.
if call(foo, [a,
    b
)
    def bar():
        pass


# F-strings uses normal list parsing, so test those as well
if call(f"hello {x
    def bar():
        pass


if call(f"hello
    def bar():
        pass


try:
    ...
except:
    ...

try:
    ...
except Exception1 as e:
    ...
except Exception2 as e:
    ...

try:
    ...
except Exception as e:
    ...
except:
    ...
finally:
    ...

try:
    ...
except:
    ...
else:
    ...

try:
    ...
except:
    ...
else:
    ...
finally:
    ...

try:
    ...
finally:
    ...

try:
    ...
else:
    ...
finally:
    ...

try:
    ...
except* GroupA as eg:
    ...
except* ExceptionGroup:
    ...

try:
    raise ValueError(1)
except TypeError as e:
    print(f"caught {type(e)}")
except OSError as e:
    print(f"caught {type(e)}")

try:
    raise ExceptionGroup("eg", [ValueError(1), TypeError(2), OSError(3), OSError(4)])
except* TypeError as e:
    print(f"caught {type(e)} with nested {e.exceptions}")
except* OSError as e:
    print(f"caught {type(e)} with nested {e.exceptions}")

try:
    pass
except "exception":
    pass
except 1:
    pass
except True:
    pass
except 1 + 1:
    pass
except a | b:
    pass
except x and y:
    pass
except await x:
    pass
except lambda x: x:
    pass
except x if True else y:
    pass

if True:
    try:
        pass
    finally:
        pass
# This `else` is not part of the `try` statement, so don't raise an error
else:
    pass



{y for y in (1, 2, 3)}
{x1: x2 for y in z}
{x + 1: 'x' for i in range(5)}
{b: c * 2 for c in d if x in w if y and yy if z}
{a: a ** 2 for b in c if d and e for f in j if k > h}
{a: b for b in c if d and e async for f in j if k > h}
{a: a for b, c in d}

# Non-parenthesized iter/if for the following expressions aren't allowed, so make sure
# it parses correctly for the parenthesized cases
{x: y for x in (yield y)}
{x: y for x in (yield from y)}
{x: y for x in (lambda y: y)}
{x: y for x in data if (yield y)}
{x: y for x in data if (yield from y)}
{x: y for x in data if (lambda y: y)}



# parse_options: {"target-version": "3.15"}
try:
    lazy import os
except:
    pass

try:
    x
except* Exception:
    lazy import sys

def func():
    lazy import math

async def async_func():
    lazy from json import loads

class MyClass:
    lazy import typing

def outer():
    class Inner:
        lazy import json



# Cases sampled from Lib/test/test_patma.py

# case test_patma_098
match x:
    case -0j:
        y = 0
# case test_patma_142
match x:
    case bytes(z):
        y = 0
# case test_patma_073
match x:
    case 0 if 0:
        y = 0
    case 0 if 1:
        y = 1
# case test_patma_006
match 3:
    case 0 | 1 | 2 | 3:
        x = True
# case test_patma_049
match x:
    case [0, 1] | [1, 0]:
        y = 0
# case black_check_sequence_then_mapping
match x:
    case [*_]:
        return "seq"
    case {}:
        return "map"
# case test_patma_035
match x:
    case {0: [1, 2, {}]}:
        y = 0
    case {0: [1, 2, {}] | True} | {1: [[]]} | {0: [1, 2, {}]} | [] | "X" | {}:
        y = 1
    case []:
        y = 2
# case test_patma_107
match x:
    case 0.25 + 1.75j:
        y = 0
# case test_patma_097
match x:
    case -0j:
        y = 0
# case test_patma_007
match 4:
    case 0 | 1 | 2 | 3:
        x = True
# case test_patma_154
match x:
    case 0 if x:
        y = 0
# case test_patma_134
match x:
    case {1: 0}:
        y = 0
    case {0: 0}:
        y = 1
    case {**z}:
        y = 2
# case test_patma_185
match Seq():
    case [*_]:
        y = 0
# case test_patma_063
match x:
    case 1:
        y = 0
    case 1:
        y = 1
# case test_patma_248
match x:
    case {"foo": bar}:
        y = bar
# case test_patma_019
match (0, 1, 2):
    case [0, 1, *x, 2]:
        y = 0
# case test_patma_052
match x:
    case [0]:
        y = 0
    case [1, 0] if (x := x[:0]):
        y = 1
    case [1, 0]:
        y = 2
# case test_patma_191
match w:
    case [x, y, *_]:
        z = 0
# case test_patma_110
match x:
    case -0.25 - 1.75j:
        y = 0
# case test_patma_151
match (x,):
    case [y]:
        z = 0
# case test_patma_114
match x:
    case A.B.C.D:
        y = 0
# case test_patma_232
match x:
    case None:
        y = 0
# case test_patma_058
match x:
    case 0:
        y = 0
# case test_patma_233
match x:
    case False:
        y = 0
# case test_patma_078
match x:
    case []:
        y = 0
    case [""]:
        y = 1
    case "":
        y = 2
# case test_patma_156
match x:
    case z:
        y = 0
# case test_patma_189
match w:
    case [x, y, *rest]:
        z = 0
# case test_patma_042
match x:
    case (0 as z) | (1 as z) | (2 as z) if z == x % 2:
        y = 0
# case test_patma_034
match x:
    case {0: [1, 2, {}]}:
        y = 0
    case {0: [1, 2, {}] | False} | {1: [[]]} | {0: [1, 2, {}]} | [] | "X" | {}:
        y = 1
    case []:
        y = 2
# case test_patma_123
match (0, 1, 2):
    case 0, *x:
        y = 0
# case test_patma_126
match (0, 1, 2):
    case *x, 2,:
        y = 0
# case test_patma_151
match x,:
    case y,:
        z = 0
# case test_patma_152
match w, x:
    case y, z:
        v = 0
# case test_patma_153
match w := x,:
    case y as v,:
        z = 0

match x:
    # F-strings aren't allowed as patterns but it's a soft syntax error in Python.
    case f"{y}":
        pass
match {"test": 1}:
    case {
        **rest,
    }:
        print(rest)
match {"label": "test"}:
    case {
        "label": str() | None as label,
    }:
        print(label)
match x:
    case [0, 1,]:
        y = 0
match x:
    case (0, 1,):
        y = 0
match x:
    case (0,):
        y = 0
match x,:
    case z:
        pass
match x, y:
    case z:
        pass
match x, y,:
    case z:
        pass

# PatternMatchSingleton
match x:
    case None:
        ...
    case True:
        ...
    case False:
        ...

# PatternMatchValue
match x:
    case a.b:
        ...
    case a.b.c:
        ...
    case '':
        ...
    case b'':
        ...
    case 1:
        ...
    case 1.0:
        ...
    case 1.0J:
        ...
    case 1 + 1j:
        ...
    case -1:
        ...
    case -1.:
        ...
    case -0b01:
        ...
    case (1):
        ...

# PatternMatchOr
match x:
    case 1 | 2:
        ...
    case '' | 1.1 | -1 | 1 + 1j | a.b:
        ...

# PatternMatchAs
match x:
    case a:
        ...
match x:
    case a as b:
        ...
match x:
    case 1 | 2 as two:
        ...
    case 1 + 3j as sum:
        ...
    case a.b as ab:
        ...
    case _ as x:
        ...
match x:
    case _:
        ...

# PatternMatchSequence
match x:
    case 1, 2, 3:
        ...
    case (1, 2, 3,):
        ...
    case (1 + 2j, a, None, a.b):
        ...
    case (1 as X, b) as S:
        ...
    case [1, 2, 3 + 1j]:
        ...
    case ([1,2], 3):
        ...
    case [1]:
        ...

# PatternMatchStar
match x:
    case *a,:
        ...
    case *_,:
        ...
    case [1, 2, *rest]:
        ...
    case (*_, 1, 2):
        ...

# PatternMatchClass
match x:
    case Point():
        ...
    case a.b.Point():
        ...
    case Point2D(x=0):
        ...
    case Point2D(x=0, y=0,):
        ...
    case Point2D(0, 1):
        ...
    case Point2D([0, 1], y=1):
        ...
    case Point2D(x=[0, 1], y=1):
        ...

# PatternMatchMapping
match x := b:
    case {1: _}:
        ...
    case {'': a, None: (1, 2), **rest}:
        ...

# Pattern guard
match y:
    case a if b := c: ...
    case e if  1 < 2: ...

# `match` as an identifier
match *a + b, c   # ((match * a) + b), c
match *(a + b), c   # (match * (a + b)), c
match (*a + b, c)   # match ((*(a + b)), c)
match -a * b + c   # (match - (a * b)) + c
match -(a * b) + c   # (match - (a * b)) + c
match (-a) * b + c   # (match (-(a * b))) + c
match ().a   # (match()).a
match (()).a   # (match(())).a
match ((),).a   # (match(())).a
match [a].b   # (match[a]).b
match [a,].b   # (match[(a,)]).b  (not (match[a]).b)
match [(a,)].b   # (match[(a,)]).b
match()[a:
    b]  # (match())[a: b]
if match := 1: pass
match match:
    case 1: pass
    case 2:
        pass
match = lambda query: query == event
print(match(12))



# Invalid target
{x: y for 1 in y}
{x: y for 'a' in y}
{x: y for call() in y}
{x: y for {a, b} in y}

# Invalid iter
{x: y for x in *y}
{x: y for x in yield y}
{x: y for x in yield from y}
{x: y for x in lambda y: y}

# Invalid if
{x: y for x in data if *y}
{x: y for x in data if yield y}
{x: y for x in data if yield from y}
{x: y for x in data if lambda y: y}


def d[T]() -> (await 1): ...
def e[T](arg: (await 1)): ...
def f[T]() -> (y := 3): ...
def g[T](arg: (x := 1)): ...
def h[T](x: (yield 1)): ...
def j[T]() -> (yield 1): ...
def l[T](x: (yield from 1)): ...
def n[T]() -> (yield from 1): ...
def p[T: (yield 1)](): ...      # yield in TypeVar bound
def q[T = (yield 1)](): ...     # yield in TypeVar default
def r[*Ts = (yield 1)](): ...   # yield in TypeVarTuple default
def s[**Ts = (yield 1)](): ...  # yield in ParamSpec default
def t[T: (x := 1)](): ...       # named expr in TypeVar bound
def u[T = (x := 1)](): ...      # named expr in TypeVar default
def v[*Ts = (x := 1)](): ...    # named expr in TypeVarTuple default
def w[**Ts = (x := 1)](): ...   # named expr in ParamSpec default
def t[T: (await 1)](): ...       # await in TypeVar bound
def u[T = (await 1)](): ...      # await in TypeVar default
def v[*Ts = (await 1)](): ...    # await in TypeVarTuple default
def w[**Ts = (await 1)](): ...   # await in ParamSpec default



from typing import TypedDict, type_check_only
from typing_extensions import NotRequired, TypeAlias

__all__ = ["parse"]

_TypeIgnores: TypeAlias = list[tuple[int, list[str]]]

@type_check_only
class ParseError(TypedDict):
    line: int
    column: int
    message: str
    blocker: NotRequired[bool]
    code: NotRequired[str]

@type_check_only
class _ASTData(TypedDict):
    is_partial_package: bool
    uses_template_strings: bool
    mypy_ignores: _TypeIgnores
    source_hash: str
    mypy_comments: list[tuple[int, str]]

def parse(
    fnam: str,
    source: str | bytes | None = None,
    skip_function_bodies: bool = False,
    python_version: tuple[int, int] | None = None,
    platform: str | None = None,
    always_true: list[str] | None = None,
    always_false: list[str] | None = None,
    cache_version: int = 0,
) -> tuple[bytes, list[ParseError], _TypeIgnores, bytes, _ASTData]:
    ...



# raise
raise
raise a
raise (a, b)
raise 1 < 2
raise a and b
raise lambda x: y
raise await x
raise x if True else y

# raise ... from ...
raise x from a
raise x from (a, b)
raise x from 1 < 2
raise x from a and b
raise x from lambda x: y
raise x from await x
raise x from x if True else y



@function_decorator
def test():
    pass


@class_decorator
class Test:
    pass


@decorator
def f(): ...


@a.b.c
def f(): ...


@a
@a.b.c
def f(): ...


@a
@1 | 2
@a.b.c
class T: ...


@x := 1
@x if True else y
@lambda x: x
@x and y
@(yield x)
@(*x, *y)
def f(): ...


# This is not multiple decorators on the same line but rather a binary (`@`) expression
@x @y
def foo(): ...


@x


@y


def foo(): ...


# Invalid keyword pattern in class argument
match subject:
    case Foo(x as y = 1):
        pass
    case Foo(x | y = 1):
        pass
    case Foo([x, y] = 1):
        pass
    case Foo({False: 0} = 1):
        pass
    case Foo(1=1):
        pass
    case Foo(Bar()=1):
        pass
    # Positional pattern cannot follow keyword pattern
    # case Foo(x, y=1, z):
    #     pass



# Double star means that the parser will try to parse it as a dictionary expression but
# it's actually a comprehension.

{**x: y for x, y in data}

# TODO(dhruvmanila): This test case fails because there's no way to represent `**y`
# in the AST. The parser tries to parse it as a binary expression but the range isn't
# correct.
# {x: **y for x, y in data}



# Invalid target
[x for 1 in y]
[x for 'a' in y]
[x for call() in y]
[x for {a, b} in y]

# Invalid iter
[x for x in *y]
[x for x in yield y]
[x for x in yield from y]
[x for x in lambda y: y]
[**x for x in [{1: 2}]]
[*x, for x in y]
[*x, *y for x in y]

# Invalid if
[x for x in data if *y]
[x for x in data if yield y]
[x for x in data if yield from y]
[x for x in data if lambda y: y]
[*x if x else y for x in z]
[x if x else *y for x in z]



# Unary addition isn't allowed but we parse it for better error recovery.
match subject:
    case +1:
        pass
    case 1 | +2 | -3:
        pass
    case [1, +2, -3]:
        pass
    case Foo(x=+1, y=-2):
        pass
    case {True: +1, False: -2}:
        pass



type Alias[T, T] = ...
def f[T, T](t: T): ...
class C[T, T]: ...
type Alias[T, U: str, V: (str, bytes), *Ts, **P, T = default] = ...
def f[T, T, T](): ...  # two errors
def f[T, *T](): ...    # star is still duplicate
def f[T, **T](): ...   # as is double star



# parse_options: {"target-version": "3.10"}
# regression tests for https://github.com/astral-sh/ruff/issues/16874
# starred parameters are fine, just not the annotation
from typing import Annotated, Literal
def foo(*args: Ts): ...
def foo(*x: Literal["this should allow arbitrary strings"]): ...
def foo(*x: Annotated[str, "this should allow arbitrary strings"]): ...
def foo(*args: str, **kwds: int): ...
def union(*x: A | B): ...



#!/usr/bin/env python3
"""Smoke test for ast_serialize extension."""
import tempfile
import ast_serialize

def test_parse_simple_file():
    """Test that we can parse a simple Python file."""
    # Create a temporary Python file
    with tempfile.NamedTemporaryFile(mode='w', suffix='.py', delete=False) as f:
        f.write("print('hello')\n")
        fname = f.name

    try:
        # Parse the file
        result = ast_serialize.parse(fname)

        # Verify we got bytes back
        assert isinstance(result[0], bytes), f"Expected bytes, got {type(result)}"
        assert len(result[0]) > 0, "Expected non-empty result"

        print(f"Successfully parsed file: {len(result[0])} bytes")
        return True
    except Exception as e:
        print(f"Test failed: {e}")
        return False
    finally:
        import os
        os.unlink(fname)

if __name__ == '__main__':
    success = test_parse_simple_file()
    exit(0 if success else 1)



match ruff:
    case {"lint": {"select": x} | {"extend-select": x}} | {"select": x}:
        ...
match 42:
    case [[x] | [x]] | x: ...
match 42:
    case [[x | x] | [x]] | x: ...
match 42:
    case ast.Subscript(n, ast.Constant() | ast.Slice()) | ast.Attribute(n): ...
